@@ -8,7 +8,7 @@ package Devel::MAT::InternalTools;
 use strict;
 use warnings;
 
-our $VERSION = '0.37';
+our $VERSION = '0.38';
 
 package Devel::MAT::Tool::help;
 
@@ -92,6 +92,11 @@ sub help_cmd
       return;
    }
 
+   if( my $code = $tool->can( "help_cmd" ) ) {
+      $tool->$code();
+      return;
+   }
+
    my %optspec = $tool->CMD_OPTS;
    my @argspec = $tool->CMD_ARGS;
 
